export TestSolid,
    TestFluid,
    Fluid,
    Solid,
    Fluid_sets


@doc """

    Material

Abstract type which serves as the common type to every other fluid or solid type in the package.

"""
abstract type Material end

@doc """

    Solid <: Material

Abstract type which serves as a common type for every inert solid defined in the package.
"""
abstract type Solid <: Material end

@doc """

    Fluid <: Material

Abstract type which serves as a common type for every pure fluid defined in the package.
"""
abstract type Fluid <: Material end

@doc """

    TestSolid <: Solid

Type defined as a template solid with all the minimum parameters required to define new solids in the package. For all properties, the function call with this solid should trhow an error for not implemented property.

# Parameters
- `name::Symbol`: Name of the Solid. Default value: `:TestSolid`.
- `ρ::Float64`: Density of the Solid in the std condition in [kg/m³]. Default value: `1000.0`.
- `cₚ::Float64`: Specific Heat of the Solid in the std condition in [J/kg K]. Default value: `4000.0`.
- `k::Float64`: Thermal conductivity of the Solid in the std condition in [W/m K]. Default value: `0.58`.
"""
struct TestSolid <: Solid
    name::Symbol
    ρ::Float64
    cₚ::Float64
    k::Float64

    @doc """
        TestSolid()

    Constructor for the TestSolid type.
    """
    function TestSolid()
        new(
            :TestSolid,
            1000.0,
            4000.0,
            0.58
        )
    end
end

@doc """

    TestFluid <: Fluid

Type defined as a template fluid with all the minimum parameters required to define new fluids in the package. For all properties, the function call with this fluid should trhow an error for not implemented property.

# Parameters
- `name::Symbol`: Name of the Fluid. Default value: `:TestFluid`.
- `ρ::Float64`: Density of the Fluid in the std condition in [kg/m³]. Default value: `1000.0`.
- `cₚ::Float64`: Specific Heat of the Fluid in the std condition in [J/kg K]. Default value: `4000.0`.
- `μ::Flaot64`: Dynamic viscosity of the Fluid in the std condition i [Pa s]. Default value: `1.002E-3`.
- `k::Float64`: Thermal conductivity of the Fluid in the std condition in [W/m K]. Default value: `0.58`.
"""
struct TestFluid <: Fluid
    name::Symbol
    ρ::Float64
    cₚ::Float64
    μ::Float64
    k::Float64

    @doc """
        TestFluid()

    Constructor for the TestFluid type.
    """
    function TestFluid()
        new(
            :Water,
            1000.0,
            4000.0,
            1.002E-3,
            0.58
        )
    end
end




## General computation common to every fluid

@doc """

    (F::Fluid)(P,T)

General function for every subtype of fluid. This function calls every property calculation for the appropriate type at the point of `P` [Pa] and `T` [K].

# Arguments
- `P`: Pressure [Pa].
- `T`: Temperature [K].
# Output
- `output::NamedTuple`: Named tuple with all the properties calculated.
```julia
(
    ρ=density [kg/m³],
    κ=compressibility [1/Pa],
    β=expansivity [1/K],
    C=specific heat [J/kg K],
    k=thermal conductivity [W/m K],
    μ=dynamic viscosity [Pa s]
)
```
"""
function (F::Fluid)(P, T)
    ρ = rho(P, T, F)
    κ = compress(P, T, F)
    β = therm_expand(P, T, F)
    C = Cp(P, T, F)
    k = therm_cond(P, T, F)
    μ = visc(P, T, F)

    return (ρ = ρ,
        κ = κ,
        β = β,
        Cₚ = C,
        k = k,
        μ = μ)
end
